# dbus-waiter

**dbus-waiter** is a small program that implements readiness checks
for services that take D-Bus names.

## Usage

    dbus-waiter --notify-fd FD --bus-name BUS_NAME PROG [ARGS]...

The following example executes `xdg-desktop-portal`, and sends a readiness
notification on file descriptor `3` once the executed process takes the bus
named `org.freedesktop.portal.Desktop`:

    dbus-waiter \
        --notify-fd 3 \
        --bus-name org.freedesktop.portal.Desktop \
        /usr/libexec/xdg-desktop-portal

The implemented readiness notification mechanism works on both `systemd` and
`s6`.

If no file descriptor matches the value of `--notify-fd`, the output error will
include:

    Error: Invalid argument (os error 22)

## Implementation

**dbus-waiter** double-forks, and reaps the child process, leaving only the
parent and the grandchild alive. The grandchild is now an orphan and is
re-parented to `pid 1`, which ensures that it will be reaped when it exits.

The parent process will block until the grandchild signals that it is ready.
The grandchild connects to the dbus session bus, and monitors ownership of the
specified bus. It then signals to the parent that it is ready.

When the parent receives the grandchild's readiness notification, it will
`exec` into `PROG`, using `ARGS` as arguments.

The grandchild will be notified when `BUS_NAME` has a new owner:

- If the owner is the parent process, the grandchild will send a readiness
  notification through `notify-fd` and exit.

- If the owner is a different process, the grandchild will exit. In this error
  scenario, the service supervisor receives not readiness notification, and
  _should_ kill the parent process after a timeout.

## Caveats

- Attacks based on PID re-use are theoretically possible, although the
  attacker needs to have permissions to take `BUS_NAME` on the specified bus.

## See also

- https://www.skarnet.org/software/s6/notifywhenup.html
- https://github.com/chimera-linux/dbus-wait-for
  A similar tool written in C with a similar approach.

## LICENCE

This project is licensed under the ISC licence. See LICENCE for details.
